<?php
/**
 * @file
 * Contains \Drupal\webtexttool\Controller\WebtexttoolController.
 */

namespace Drupal\webtexttool\Controller;

use Drupal\Component\Utility\Random;
use Drupal\Component\Utility\SafeMarkup;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\Element;
use Drupal\metatag\Plugin\DataType\Metatag;
use Drupal\metatag\Plugin\migrate\source\d7\MetatagFieldInstance;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use Drupal\Core\Url;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

/**
 * Class WebtexttoolController
 *
 * @package Drupal\webtexttool\Controller
 */
class WebtexttoolController extends ControllerBase {

  /**
   * Base url of the webtexttool API.
   */
  const WEBTEXTTOOL_URL = 'https://api.textmetrics.com';

  /**
   * The temp store factory.
   */
  protected $tempStoreFactory;

  /**
   * Helper function to check if user is authenticated.
   */
  public function webtexttoolIsAuthenticated() {
    $is_authenticated = $this->webtexttoolConnect('/user/authenticated');
    return $is_authenticated;
  }
  /**
   * Implements searchkeyword from api.
   */
  public function webtexttoolSearchKeyword($keyword, $database = 'us', $language = 'us') {
    $response = $this->webtexttoolConnect('/keywords/' . UrlHelper::encodePath($keyword). '/'. $database . '/' . $language, $this->webtexttoolGetToken());

    return $response;
  }

  /**
   * Implements GetSuggestions from api.
   */
  public function webtexttoolAnalyseSeo($html, $keywords, $language = 'en', $ruleset, $synonyms = FALSE) {

    // Set default paramaters
    $parameters = array(
      'content' => $html,
      'keywords' => $keywords,
      'languageCode' => $language,
      'ruleset' => $ruleset,
      'synonyms' => $synonyms,
    );

    $response = $this->webtexttoolConnect('/page/suggestions',$this->webtexttoolGetToken(), 'post', $parameters);
    return $this->makeUniformResponseSeo($response);
  }

  /**
   *
   * Connects to the webtexttool API.
   *
   * @param $function
   * @param string $token
   * @param string $method
   * @param array $parameters
   *
   * @return bool|mixed|string
   */
  protected function webtexttoolConnect($function, $token = '',$method = 'get', $parameters = array()) {

    // If we don't have a token, get one.
    if($token == '') {
      $token = $this->webtexttoolLogin();
    }
    try {
      // Construct headers and client.
      $headers = array('Content-Type' => 'application/json', 'WttSource' => 'Drupal', 'Authorization' => 'bearer ' . $token);
      $client = new Client([
        'base_uri' => SELF::WEBTEXTTOOL_URL,
        'headers' => $headers
      ]);

      // Post handling.
      if($method == 'post'){
        $request = $client->post($function, [
          'body' => json_encode($parameters),
        ]);
      }
      // Get handling.
      else{
        $request = $client->get($function);
      }

      // Get the response body of the request.
      $data = (string) $request->getBody();

      // Return if the reponse is empty.
      if (empty($data)) {
        return FALSE;
      }

      // Return the result of the request.
      else{
        $data = json_decode($data);
        if(!empty($data)) {
          return $data;
        }
        else{
          return false;
        }
      }
    }
    // Error handling.
    catch (RequestException $e) {
      drupal_set_message(t('Unable to connect to the Textmetrics service please check your credentials and try again later.'), 'error');
      return FALSE;
    }

    return false;
  }

  /**
   * Register to webtexttool api service.
   *
   */
  public function webtexttoolRegister($email, $pass, $language) {

    // Construct headers and request body.
    $headers = array('Content-Type' => 'application/json', 'WttSource' => 'Drupal');
    $body = array(
      'Language' => $language,
      'UserName' => $email,
      'Password' => $pass,
    );
    try {

      // Construct client.
      $client = new Client([
        'base_uri' => SELF::WEBTEXTTOOL_URL,
      ]);

      // Create the request.
      $request = $client->post('/user/register', [
        'headers' => $headers,
        'body' => json_encode($body),
      ]);

      // Get the response body.
      $data = (string) $request->getBody();

      // Jump ship if we don't have a response body.
      if (empty($data)) {
        return FALSE;
      }
      else {

        // Decode the response body.
        $data = json_decode($data);
        $token = $data->access_token;

        // If we received an access token, save it.
        if(!empty($token)){
          $this->webtexttoolSetToken($token);
          return $token;
        }
        else{
          return false;
        }
      }
    }
    // Error handling.
    catch (RequestException $e) {
      return FALSE;
    }
  }

  /**
   * Logs in to webtexttool api service.
   *
   */
  public function webtexttoolLogin() {

    // Create headers and response body.
    $headers = array('Content-Type' => 'application/json', 'WttSource' => 'Drupal');
    $body = array(
      'Language' => \Drupal::config('webtexttool.settings')->get('language'),
      'RememberMe' => TRUE,
      'UserName' => \Drupal::config('webtexttool.settings')->get('user'),
      'Password' => \Drupal::config('webtexttool.settings')->get('pass'),
    );
    try {

      // Construct client and request.
      $client = new Client([
        'base_uri' => SELF::WEBTEXTTOOL_URL,
      ]);

      $request = $client->post("/user/login", [
        'headers' => $headers,
        'body' => json_encode($body),
        'timeout' => 60,
      ]);

      // Check if we have a response body.
      $data = (string) $request->getBody();
      if (empty($data)) {
        return FALSE;
      }
      else{

        // Decode the response.
        $data = json_decode($data);
        $token = $data->access_token;

        // If we received an access token, save it.
        if(!empty($token)){
          $this->webtexttoolSetToken($token);
          return $token;
        }

        // Give feedback about the failed login.
        else {
          drupal_set_message(t('Unable to login to the Textmetrics service please check your credentials and try again later.'), 'error');
          return false;
        }
      }
    }
    // Error handling.
    catch (RequestException $e) {
      drupal_set_message(t('Unable to login to the Textmetrics service please check your credentials and try again later.'), 'error');
      return FALSE;
    }
  }

  /**
   * Generate the correct headers with bearer.
   *
   */
  protected function webtexttoolHeaders($token = '') {

    // Set up headers.
    $headers = array('Content-Type' => 'application/json', 'WttSource' => 'Drupal');

    // Get the token
    $token = $this->webtexttoolGetToken($token);

    // Set up authorization header.
    if($token && $token !== ''){
      $headers['Authorization'] = 'bearer ' . $token;
    }

    // Return the headers.
    return $headers;
  }

  /**
   * Set token from webtexttool.
   */
  public function webtexttoolSetToken($token) {

    // Set the received token in the temporary storage.
    $tempstore = \Drupal::service('user.private_tempstore')->get('webtexttool');
    $tempstore->set('webtexttooltoken', $token);
  }

  /**
   * Get token from webtexttool.
   */
  protected function webtexttoolGetToken($token = '') {

    // Jump ship if we already have a token.
    if($token !== ''){
      return $token;
    }

    // Get the token form the temporary storage.
    $tempstore = \Drupal::service('user.private_tempstore')->get('webtexttool');
    $token = $tempstore->get('webtexttooltoken');

    // If we don't have a token in the temporary storage, get a new one.
    if(empty($token)) {
      $token = $this->webtexttoolLogin();
    }

    // Return the token.
    return $token;
  }

  /**
   * Implements getkeywordsources from api.
   */
  public function webtexttoolGetSources() {

    $keywords = array();

    // Get the keyword sources from the cache.
    if ($cache = \Drupal::cache()->get('webtexttool_getkeywordsources') ) {
      $keywords = $cache->data;
    }
    // Get the list of languages supported.
    else {
      $keywords_languages = $this->webtexttoolConnect('/keywords/sources');

      if ($keywords_languages) {
        foreach ($keywords_languages as $keyword) {
          $keywords[$keyword->Value] = $keyword->Name;
        }

        // Set the keyword sources in the cache.
        $set_cache = \Drupal::cache()->set('webtexttool_getkeywordsources', $keywords);
      }
    }

    // Return the keywords.
    return $keywords;
  }

  /**
   * Page callback for the account status page.
   */
  public function accountStatus() {

    // Check if the user is authenticated.
    $is_authenticated = $this->webtexttoolIsAuthenticated();

    // If not, try to login the user.
    if (!$is_authenticated) {
      $this->webtexttoolLogin();
    }

    // Return the markup of the account.
    $render = array(
      '#markup' => $this->webtexttoolAccount(),
    );

    return $render;
  }


  /**
   * Get an account.
   */
  public function webtexttoolAccount() {

    // Get the current account.
    $account = $this->webtexttoolConnect('/user/info', $this->webtexttoolGetToken());

    if ($account) {

      // Create a render array.
      $render = array(
        '#prefix' => '<div class="webtexttool-account">',
        'account' => array(
          '#markup' => 'Username: ' . $account->UserName . ' FullName: ' . $account->FullName . ' Subscription: ' . $account->SubscriptionName,
        ),
        '#suffix' => '</div>'
      );

      return render($render);
    }

    // Give feedback that the user is not currently logged in.
    $url = Url::fromRoute('system.admin_config_webtexttool.settings');
    return $this->t('Unable to load your account. Please check and update your credentials at @url.', array('@url' => $this->l($this->t('My account'), $url)));
  }

  /**
   * Returns a preset of the response of the node. Either an earlier saved response, or a dummy response to build the
   * Form structure of the content quality tab.
   */
  public function getResponsePreset($node = FALSE) {
    // Check if we have a viable node.
    if ($node->id()) {
      $record = \Drupal::database()->select('webtexttool', 'wtt')
        ->condition('nid', $node->id())
        ->fields('wtt', array('content_quality_response'))
        ->execute();

      $earlier_response = '';
      while ($row = $record->fetchAssoc()) {
        $earlier_response = $row['content_quality_response'];
      }

      // Decode the response.
      $response = json_decode($earlier_response, TRUE);
    }

    // Return the decoded resonse if available, or a dummy preset if not.
    if (!empty($response)) {
      return $response;
    }
    else {
      return $this->getDummyUniformResponseContentQuality();
    }
  }

  /**
   * Returns a dummy response for the content quality check.
   */
  private function getDummyUniformResponseContentQuality() {

    $dummy_return_response = array(
      'original' => array(
        'Suggestions' => array(
          'PageScore' => 0,
          'Suggestions' => array(),
          'PageScoreTag' => '',
          'RuleSet' => 'ContentQuality',
        ),
        'ModifiedDate' => '',
        'Details' => array(
          'ReadingTime' => '00:00',
          'ReadingLevel' => '',
          'ReadingValues' => array(),
        ),
      ),
      'title' => 'Content Quality',
      'suggestions' => array(
        array(
          'Tag' => 'Readability',
          'MetaTag' => 'Readability',
          'Rules' => array(),
          'Importance' => 0,
          'Score' => 0,
          'Penalty' => 0,
          'Tooltip' => NULL,
          'SortIndex' => 0,
        ),
        array(
          'Tag' => 'Adjectives',
          'MetaTag' => 'Adjectives',
          'Rules' => array(),
          'Importance' => 0,
          'Score' => 0,
          'Penalty' => 0,
          'Tooltip' => NULL,
          'SortIndex' => 0,
        ),
        array(
          'Tag' => 'Whitespaces',
          'MetaTag' => 'Whitespaces',
          'Rules' => array(),
          'Importance' => 0,
          'Score' => 0,
          'Penalty' => 0,
          'Tooltip' => NULL,
          'SortIndex' => 0,
        ),
        array(
          'Tag' => 'Gender',
          'MetaTag' => 'Gender',
          'Rules' => array(),
          'Importance' => 0,
          'Score' => 0,
          'Penalty' => 0,
          'Tooltip' => NULL,
          'SortIndex' => 0,
        ),
        array(
          'Tag' => 'Sentiment',
          'MetaTag' => 'Sentiment',
          'Rules' => array(),
          'Importance' => 0,
          'Score' => 0,
          'Penalty' => 0,
          'Tooltip' => NULL,
          'SortIndex' => 0,
        ),
      ),
      'stats' => array(
        'Reading time' => '00:00',
        'Reading level' => t('Not analyzed yet')
      ),
    );

    $account = $this->webtexttoolConnect('/user/info', $this->webtexttoolGetToken());
    if (in_array('AdvancedLanguageLevel', $account->Features) !== FALSE) {

      // Add advanced reading level preset here.
      $dummy_return_response["suggestions"][0]['Metadata'] = array(
        'Settings' => array(
          array(
            'DisplayName' => 'A1',
            'Value' => 1,
          ),
          array(
            'DisplayName' => 'A2',
            'Value' => 12,
          ),
          array(
            'DisplayName' => 'B1',
            'Value' => 2,
          ),
          array(
            'DisplayName' => 'B2',
            'Value' => 14,
          ),
          array(
            'DisplayName' => 'C1',
            'Value' => 3,
          ),
          array(
            'DisplayName' => 'C2',
            'Value' => 16,
          ),
        ),
      );
    }

    return $dummy_return_response;
  }

  /**
   * Post-process the suggestion.
   */
  public function postProcessExtraInfoObject($suggestion) {
    if (isset($suggestion['Rules']) && count($suggestion['Rules'])) {
      foreach ($suggestion['Rules'] as $delta => $rule) {

        if (isset($rule['ExtraInfo'])) {
          $extra_info_decoded = json_decode($rule['ExtraInfo'], TRUE);

          // Remove blacklisted words that are Drupal specific.
          if (isset($extra_info_decoded["List"]) && count($extra_info_decoded["List"])) {
            foreach ($extra_info_decoded["List"] as $list_delta => $item) {

              if (in_array($item['word'], $this->getBlacklistedWords())) {
                unset($extra_info_decoded['List'][$list_delta]);
              }

              if (strpos($item['word'], 'pager]Drupal') !== FALSE) {
                unset($extra_info_decoded['List'][$list_delta]);
              }
            }
          }

          $suggestion['Rules'][$delta]['ExtraInfo'] = $extra_info_decoded;
        }
      }
    }

    return $suggestion;
  }

  /**
   * Implements searchkeyword from api.
   */
  public function analyseContentQuality($html, $settings = array(), $language = 'en') {

    $readability = 0;
    if (isset($settings['toggle_Readability']) && $settings['toggle_Readability'] == TRUE && isset($settings['Readability']['readability_setting'])) {
      $readability = $settings['Readability']['readability_setting'];
    }

    $content_quality_parameters = array(
      'QualityLevels' => array(
        // Readability can be 0, 1, 2, 3
        'ReadingLevel' => $readability,
        'DifficultWordsLevel'  => $readability,
        'LongSentencesLevel'  => $readability,

        // toggle_Adjectives triggers two items to call.
        'AdjectivesLevel' => isset($settings['toggle_Adjectives']) ? $settings['toggle_Adjectives'] : 1,
        'AdjectivesList' => isset($settings['toggle_Adjectives']) ? $settings['toggle_Adjectives'] : 1,
        'BulletPointsLevel' => isset($settings['toggle_Bulletpoints']) ? $settings['toggle_Bulletpoints'] : 1,
        'WhitespacesLevel' => isset($settings['toggle_Whitespaces']) ? $settings['toggle_Whitespaces'] : 1,
      ),
      'content' => $html,
      'languageCode' => $language,
      'ContentLanguageCode' => null,
      'SmartSearch' => FALSE,
      'RuleSet' => 20
    );

    if (isset($settings['toggle_Gender']) && $settings['toggle_Gender'] == 1) {
      // GenderLevel can be 'm', 'f' or 'n'
      $content_quality_parameters['QualityLevels']['GenderLevel'] = isset($settings['Gender']['gender_setting']) ? $settings['Gender']['gender_setting'] : 'n';
    }

    if (isset($settings['toggle_Sentiment']) && $settings['toggle_Sentiment'] == 1) {
      // SentimentLevel can be 'positive', 'neutral' or 'negative'
      $content_quality_parameters['QualityLevels']['SentimentLevel'] = isset($settings['Sentiment']['sentiment_setting']) ? $settings['Sentiment']['sentiment_setting'] : 'neutral';
    }

    $content_quality_response = $this->webtexttoolConnect('/contentquality/suggestions', $this->webtexttoolGetToken(), 'post', $content_quality_parameters);

    return $this->makeUniformResponseContentQuality($content_quality_response);
  }

  /**
   * Returns a unified seo response.
   */
  private function makeUniformResponseSeo($response) {
    $response = json_decode(json_encode($response),true);
    $return_response = array(
      'original' => $response,
      'title' => 'SEO',
      'suggestions' => $response["Suggestions"],
      'stats' => array(
        $this->t('Page score')->render() => round($response['PageScore']),
        $this->t('Page score potential')->render() => $response['PageScorePotential'],
        $this->t('Encouragement')->render() => $response['PageScoreTag'],
        $this->t('Keyword count')->render() => $response['KeywordCount'],
        $this->t('Word count')->render() => $response['WordCount'],
      )
    );

    return $return_response;
  }

  /**
   * Returns a unified content quality response.
   */
  private function makeUniformResponseContentQuality ($response) {

    $response_array = json_decode(json_encode($response), TRUE);

    $return_response = array(
      'original' => $response_array,
      'title' => 'Content Quality',
      'suggestions' => $response_array["Suggestions"]["Suggestions"],
      'stats' => array(
        'Reading time' => $response_array["Details"]["ReadingTime"],
        'Reading level' => $response_array["Details"]["ReadingLevel"],
      ),
      'page_score' => $response_array['Suggestions']['PageScore'],
      'page_score_tag' => $response_array['Suggestions']['PageScoreTag'],
    );

    if (isset($response_array["Details"]["Sentiment"])) {
      $return_response['stats']['Sentiment'] = $response_array["Details"]["Sentiment"];
    }

    return $return_response;
  }


  /**
   * Returns a unified content quality settings structure so we can set in on the node.
   */
  public function getUnifiedContentQualitySettings($form_state) {

    $values = $form_state->getValues();

    $unified_content_quality = array(
      'toggle_Readability' => $values['toggle_Readability'],
      'Readability' => array(
        'readability_setting' => $values['readability_setting'],
      ),
      'toggle_Adjectives' => $values['toggle_Adjectives'],
      'toggle_Whitespaces' => $values['toggle_Whitespaces'],
      'toggle_Gender' => $values['toggle_Gender'],
      'Gender' => array(
        'gender_setting' => $values['gender_setting'],
      ),
      'toggle_Sentiment' => $values['toggle_Sentiment'],
      'Sentiment'=> array(
        'sentiment_setting' => $values['sentiment_setting'],
      ),
    );

    return $unified_content_quality;
  }

  /**
   * Encodes and saves the content quality response of the node.
   */
  public function saveContentQualityResponse($node, $response) {

    // Check if we have a response and the CQC settings on the node are set.
    if (!empty($response) && !empty($node->webtexttool_content_quality_settings) && isset($node->nid) && $node->nid) {
      if (isset($response['original'])) {
        unset($response['original']);
      }

      $response_encoded = json_encode($response);

      // First check if there is already an db entry for this node.
      if (isset($node->nid->value)) {
        $connection = \Drupal::database();
        $query = $connection->query("SELECT nid FROM {webtexttool} WHERE nid = " . $node->nid->value);
        $result = $query->fetchAll();

        if (empty($result)) {
          // Insert a row in the webtexttool table for this node.
          $connection->insert('webtexttool')
            ->fields(array(
              'nid' => $node->nid->value,
              'vid' => $node->vid->value,
              'content_quality_response' => $response_encoded
            ))
            ->execute();
        }
        else {
          // Update the row and inject the encoded response into the webtextool table.
          $connection->merge('webtexttool')
            ->key(array('nid' => $node->nid->value))
            ->fields(array(
              'content_quality_response' => $response_encoded
            ))
            ->execute();
        }
      }
    }
  }

  /**
   * Save the content quality settings on the node.
   */
  public function saveContentQualitySettings($node, $response) {

    // Check if we have a response and the CQC settings on the node are set.
    if (!empty($response) && !empty($node->webtexttool_content_quality_settings) && isset($node->nid) && $node->nid) {
      $content_quality_settings = json_encode($node->webtexttool_content_quality_settings);

      // First check if there is already an db entry for this node.
      if (isset($node->nid->value)) {
        $connection = \Drupal::database();
        $query = $connection->query("SELECT nid FROM {webtexttool} WHERE nid = " . $node->nid->value);
        $result = $query->fetchAll();

        if (empty($result)) {
          // Insert a row in the webtexttool table for this node.
          $connection->insert('webtexttool')
            ->fields(array(
              'nid' => $node->nid->value,
              'vid' => $node->vid->value,
              'content_quality_settings' => $content_quality_settings
            ))
            ->execute();
        }
        else {
          // Update the row and inject the encoded response into the webtextool table.
          $connection->merge('webtexttool')
            ->key(array('nid' => $node->nid->value))
            ->fields(array(
              'content_quality_settings' => $content_quality_settings
            ))
            ->execute();
        }
      }
    }
  }

  /**
   * Saves the initial settings.
   */
  public function saveWebtexttoolSettingsOfNode($node, $settings) {
    $connection = \Drupal::database();
    $query = $connection->query('SELECT nid, keywords, lang, content_quality_settings, ruleset, synonyms FROM {webtexttool} WHERE vid IN (:vid)', array(':vid' => $node->vid->value));
    $result = $query->fetchAll();

    // Only save the settings in the db, if the node is already saved and has an identifier.
    if (isset($node->nid->value)) {
      if (empty($result)) {
        $connection->insert('webtexttool')
          ->fields(array(
            'nid' => $node->nid->value,
            'vid' => $node->vid->value,
            'keywords' => isset($settings['webtexttool_keywords']) ? $settings['webtexttool_keywords'] : '',
            'lang' => isset($settings['webtexttool_language']) ? $settings['webtexttool_language'] : '',
            'content_quality_settings' => isset($settings['webtexttool_content_quality_settings']) ? json_encode($settings['webtexttool_content_quality_settings']) : '',
            'ruleset' => isset($settings['webtexttool_ruleset']) ? $settings['webtexttool_ruleset'] : 1,
            'synonyms' => isset($settings['webtexttool_synonyms']) ? $settings['webtexttool_synonyms'] : '',
          ))
          ->execute();
      }
      else {
        $connection->merge('webtexttool')
          ->key(array('vid' => $node->vid->value))
          ->fields(array(
            'nid' => $node->nid->value,
            'vid' => $node->vid->value,
            'keywords' => isset($settings['webtexttool_keywords']) ? $settings['webtexttool_keywords'] : '',
            'lang' => isset($settings['webtexttool_language']) ? $settings['webtexttool_language'] : '',
            'content_quality_settings' => isset($settings['webtexttool_content_quality_settings']) ? json_encode($settings['webtexttool_content_quality_settings']) : '',
            'ruleset' => isset($settings['webtexttool_ruleset']) ? $settings['webtexttool_ruleset'] : 1,
            'synonyms' => isset($settings['webtexttool_synonyms']) ? $settings['webtexttool_synonyms'] : '',
          ))
          ->execute();
      }
    }
  }

  /**
   * Returns the synonym word collection that can be used in the API call.
   */
  public function seoGetSynonymsStructureForApiCall($synonyms_value) {
    $synonyms = [];
    $synonyms_exploded = preg_split("/[\n\r]/", $synonyms_value);
    $synonyms_exploded_cleaned = array_filter($synonyms_exploded);

    $counter = 0;
    foreach ($synonyms_exploded_cleaned as $delta => $synonym) {
      if ($counter < 20) {
        $synonyms[$counter] = array('text' => $synonym);
        $counter++;
      }
    }

    return $synonyms;
  }

  /**
   * Provides markup for the given node, respecting metatags and path, with head and body tags.
   */
  public function constructCompleteMarkupOfPageWithNodeMarkup($node, $form, $form_state) {

    if (!isset($node->nid->value)) {
      $node->in_preview = TRUE;
    }

    // Get the default node markup.
    $node_render = \Drupal::entityTypeManager()->getViewBuilder('node')->view($node, 'full');
    $node_markup = \Drupal::service('renderer')->renderRoot($node_render);

    // First, get the field name of the metatags field (the name can be anything)
    $values = $form_state->getValues();
    $metatag_field_name = '';

    foreach ($values as $field_key => $field_value) {
      if (strpos($field_key, 'field_') !== FALSE) {
        if (isset($field_value[0]['basic']['title'])) {
          $metatag_field_name = $field_key;
        }
      }
    }

    // Render all metatags by hand.
    $head = '';
    if (isset($node->{$metatag_field_name}) && $values[$metatag_field_name][0]['basic']) {
      $metatags = $values[$metatag_field_name][0]['basic'];

      foreach ($metatags as $meta_key => $meta_value) {
        if ($meta_key == 'title') {
          $head .= '<title>' . strip_tags($meta_value) . '</title>';
        }
        else {
          $head .= '<meta name="' . SafeMarkup::checkPlain($meta_key) . '" content="' . htmlspecialchars(strip_tags($meta_value)) . '" /> ';
        }
      }
    }
    else {
      $title = $node->getTitle();
      $head .= '<title>' . strip_tags($title) . '</title>';

      // Let the user know there is no metatag field available, while the metatag module is enabled (and a dependency).
      drupal_set_message('To analyze the page description, add a metatag field to the current content type.', 'warning');
    }

    // Render the custom <meta name="url" content="">.
    $path = '';
    global $base_url;

    if (\Drupal::moduleHandler()->moduleExists('path') && isset($node->path) && $node->path->get(0)->alias != '') {
      $path = $node->path->get(0)->alias;
      $alias_clean = str_replace(array('-', '/'), ' ', $path);
      $path = $base_url . '/' . $alias_clean;
    }

    if ($path != '') {
      $head .= '<meta name="url" content="' . $path . '" /> ';
    }

    // Get a very plain version of the html.
    $page = '<html><head>' . $head;
    $page .= '</head><body>' . $node_markup . '</body></html>';

    return $page;
  }

  /**
   * Returns blacklisted words.
   */
  private function getBlacklistedWords() {
    return array(
      'current',
      'pager]Drupal'
    );
  }

  /**
   * Returns string used in the Wordpress plugin
   */
  public function webtexttoolHeading($key) {
    $texts = array(
      "CQGenericError" => t("We’re sorry, we could not analyze your content. Please try again or contact support@textmetrics.com in case the issues persist."),
      "ContentRequiredError" => t("Your page needs some content."),
      "ContentMinLengthError" => t("Your page content must have at least 150 words."),
      "GenericError" => t("Something went wrong!"),
      "LanguageNotSupportedError" => t("Detected language is not yet supported!"),
      "PageTitleLabel" => t("Page Title"),
      "PageDescriptionLabel" => t("Page Description"),
      "HeadingsLabel" => t("Headings"),
      "MainContentLabel" => t("Main Content"),
      "MiscellaneousLabel" => t("Miscellaneous"),
      "HeadingsSuggestion" => t("To optimize your content both in terms of readability and SEO, you should structure your content by adding several headings. At the start of your page you normally have an H1 / heading 1 with the title of your page. In some CMS&#39;s / themes this H1 is added automatically. If this is the case, you can select the option &quot;Process page title as H1&quot; from the settings above. Next to H1, you should add smaller heading (H2-H6) to structure your content even further."),
      "MainContentSuggestion" => t("Here you will find several important suggestions for your content. Please have a look at our knowledgebase (Learn tab in the app) to find more background information about these suggestions."),
      "MiscellaneousSuggestion" => t("Here you will find suggestions to optimize your content. These will have smaller impact on overall optimization, but are good to consider and see if they can fit in your content."),
      "PageTitleSuggestion" => t("<p>The Page title is important to search engines. And therefore it&#39;s important to you. Think of a catchy title that will trigger a user to click on your page when it&#39;s listed in the search results. Of course it should also cover the content of the page.</p>\r\n\t"),
      "PageDescriptionSuggestion" => t("<p>The page description is important because it&#39;s shown in the search results and it will tell the search and the users what your page is about. So provide a good description of your content and make sure you follow the suggestion for creating a perfect description of your page.</p>\r\n\t"),
      "Heading1Suggestion" => t("<p>A H1 / Header section at the beginning of your page is important because it&#39;s the readable introduction of your page. In some CMS&#39;s the Page Title is automatically inserted at the top of a page in H1/Header 1.</p>\r\n\t"),
      "Heading2to6Suggestion" => t("<p>Use smaller headings (h2, h3, h4, h5 and/or h6) in your content to highlight / summarize paragraphs. Using headers will make it easier for you reader to &quot;scan&quot; the contents of your page. It allows you to catch the reader&#39;s attention.</p>\r\n\t"),
      "BodySuggestion" => t("<p>These suggestions are related to overall content on your page. Our rules suggest a minimum number of words for your page. Also related to the length of your content, is the number of times you should use your keywords. This way you can avoid to put your keyword too many times in the content (&quot;keyword stuffing&quot;), but also make sure that you use your keyword enough times so it will be clear for the search engine what the content is about.</p>\r\n\t"),
      "ReadabilityLabel" => t("Readability"),
      "AdjectivesLabel" => t("Text credibility"),
      "GenderLabel" => t("Target audience"),
      "WhitespacesLabel" => t("Text layout"),
      "BulletpointsLabel" => t("Bulletpoints"),
      "ReadabilitySuggestion" => t("Readability => multiple checks on complexity level of the content (reading score/long sentences/difficult words)."),
      "AdjectivesSuggestion" => t("Checks the use of adjectives in your text. Over- or underuse of adjectives will decrease effectiveness of your text."),
      "GenderSuggestion" => t("Gender check on level (confidence) of content target."),
      "WhitespacesSuggestion" => t("Checks the use of white spaces in your content. Use this to make your content easier to scan and read."),
      "BulletpointsSuggestion" => t("Checks the use of bulletpoints in your content. Use these to make the text easier to scan and read.")
    );

    if (isset($texts[$key])) {
      return $texts[$key];
    }
  }

  /**
   * Helper function to add a class based on the Score.
   */
  public static function scoreToClass($score) {
    // -1 - N/A, 0 - very low, 1 - low, 2 - moderate, 3 - high, 4- very high.
    $class_array = array(
      '-1' => 'yellow',
      '0' => 'red',
      '1' => 'orange',
      '2' => 'yellow',
      '3' => 'light-green',
      '4' => 'green',
    );
    return $class_array[$score];
  }

  /**
   * Helper function to create the markup for an colord indicator.
   */
  public static function colorIndicator($color_class) {
    return "<span class='indicator $color_class'>&nbsp;</span>";
  }

  /**
   * Helper function to add a class based on the OverallScore.
   */
  public static function overallScoreToClass($score) {
    // 0 - poor, 1 - moderate, 2 - good.
    $class_array = array(
      '-1' => 'yellow',
      '0' => 'red',
      '1' => 'yellow',
      '2' => 'green',
    );
    return $class_array[$score];
  }

  /**
   * Helper function to change Competition Score to text.
   */
  public static function volumeScoreToText($score) {
    // VolumeScore: -1 - N/A, 0 - very low, 1 - low,
    // 2 - moderate, 3 - high, 4- very high.
    $score_array = array(
      '-1' => t('moderate'),
      '0' => t('very low'),
      '1' => t('low'),
      '2' => t('moderate'),
      '3' => t('high'),
      '4' => t('very high'),
    );
    return $score_array[$score];
  }

  /**
   * Helper function to change Competition Score to text.
   */
  public static function competitionScoreToText($score) {
    // -1 - N/A, 0 - very hard, 1- hard, 2 - moderate, 3 - easy, 4 - very easy.
    $score_array = array(
      '-1' => 'moderate',
      '0' => 'very hard',
      '1' => 'hard',
      '2' => 'moderate',
      '3' => 'easy',
      '4' => 'very easy',
    );
    return $score_array[$score];
  }

  /**
   * Helper function to change overall score to text.
   */
  public static function overallToText($score) {
    // 0 - poor, 1 = moderate, 2 - good.
    $score_array = array(
      '-1' => 'moderate',
      '0' => 'poor',
      '1' => 'moderate',
      '2' => 'good',
    );
    return $score_array[$score];
  }
}